/******************************************************************************/
#include "p10type.h"
#include "p10ext.h"
/******************************************************************************/
int Pre_Processor()
{
 Ck_Var();
 Open_Files();
 Read_Autocorr();
 Get_Mapping();
 Get_Inv_Mapping();
 Get_Trellis();
 Get_Rx_Pred_Coeff();
 Sim_Init();
 Print_Pred_Coeff();
 return(0);
}
/******************************************************************************/
/*                                 Open files.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("fautocor.dat","r");
 if(fp1 == NULL)
  Error_Mes(25);
 return(0);
}
/******************************************************************************/
/*                  Read the autocorrelation coefficients.
*******************************************************************************/
int Read_Autocorr()
{
 int cnt;
 double val;
 cnt=ZERO;
 printf("\t Autocorrelation coefficients without noise:\n\n");
 while(fscanf(fp1,"%lf",&val) != EOF)
 {
  if(cnt == AUTOCOR_SIZE)
   Error_Mes(26);
  Autocor[cnt]=F_VARIANCE*val;
  printf("\t Inx:%d Autocor:%18.16lf\n",cnt,Autocor[cnt]);
  cnt++;
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                           Get real mapping.
*******************************************************************************/
int Get_Mapping()
{
 int cnt;
 double tmp,inx,size;
 tmp=0.5*(2.0+(1.0+sqrt(3.0))*(1.0+sqrt(3.0)));
 tmp=sqrt(tmp);
 size=(double)CONSTELL_SIZE;
 for(cnt=ZERO;cnt<CONSTELL_SIZE;cnt++)
 {
  inx=(double)cnt;
  Re_Map[cnt]=tmp*cos(inx*2.0*MY_PI/size);
  Im_Map[cnt]=tmp*sin(inx*2.0*MY_PI/size);
 }
 return(0);
}
/******************************************************************************/
/*                     Get the inverse of the symbols.
*******************************************************************************/
int Get_Inv_Mapping()
{
 int cnt;
 double magsq;
 for(cnt=ZERO;cnt<CONSTELL_SIZE;cnt++)
 {
  magsq=Re_Map[cnt]*Re_Map[cnt]+Im_Map[cnt]*Im_Map[cnt];
  if(magsq<ERROR)
   Error_Mes(22);
  Re_Inv_Map[cnt]=  Re_Map[cnt]/magsq;
  Im_Inv_Map[cnt]= -Im_Map[cnt]/magsq;
 }
 return(0);
}
/******************************************************************************/
/*                 Print the prediction filter coefficients.
*******************************************************************************/
int Print_Pred_Coeff()
{
 int cnt;
 printf("\t Transmitter and receiver coefficients.\n\n");
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
 {
  printf("\t\t Inx:%d Tx:%+14.12lf ",cnt,Coeff_Array[cnt]);
  printf("Rx:%+14.12lf\n",Rx_Coeff_Array[cnt]);
 }
 return(0);
}
/******************************************************************************/
