/******************************************************************************/
#include "p10type.h"
#include "p10ext.h"
/******************************************************************************/
/*             Get the prediction filter coefficients at the receiver.
*******************************************************************************/
int Get_Rx_Pred_Coeff()
{
 Levinson_Durbin();
 return(0);
}
/******************************************************************************/
/*           Get the prediction coefficients from the LD algorithm.
*******************************************************************************/
int Levinson_Durbin()
{
 int order_cnt;
 Init_LD();
 for(order_cnt=ONE;order_cnt<=PRED_MEM_SIZE;order_cnt++)
 {
  Get_Reflection_Coeff(order_cnt);
  Get_New_Coeff(order_cnt);
  Get_New_Pred_Var(order_cnt);
  Print_Pred_Info(order_cnt);
 }
 Get_Rx_Coeff_Array();
 return(0);
}
/******************************************************************************/
/*                       Initialize the LD algorithm.
*******************************************************************************/
int Init_LD()
{
 int order,inx;
 double energy;
 energy=Re_Map[0]*Re_Map[0]+Im_Map[0]*Im_Map[0];
 Autocor[ZERO]=Autocor[ZERO]+VARIANCE/energy;
 Pred_Var=Autocor[ZERO];
 for(order=ZERO;order<=PRED_MEM_SIZE;order++)
 {
  Pred_Coeff[order][ZERO]=1.0;
  for(inx=ONE;inx<=PRED_MEM_SIZE;inx++)
   Pred_Coeff[order][inx]=0.0;
 }
 return(0);
}
/******************************************************************************/
/*            Get the reflection coefficient for the current order.
*******************************************************************************/
int Get_Reflection_Coeff(order_cnt)
int order_cnt;
{
 int cnt;
 double numer;
 numer=0.0;
 for(cnt=ZERO;cnt<order_cnt;cnt++)
  numer=numer+Pred_Coeff[order_cnt-ONE][cnt]*Autocor[order_cnt-cnt];
 Pred_Coeff[order_cnt][order_cnt]= -numer/Pred_Var;
 printf("\t Reflection coeff. for order %d is ",order_cnt);
 printf("%14.12lf.\n\n",Pred_Coeff[order_cnt][order_cnt]);
 return(0);
}
/******************************************************************************/
/*             Get the predictor coefficients for the next order.
*******************************************************************************/
int Get_New_Coeff(order_cnt)
int order_cnt;
{
 int cnt;
 double x;
 for(cnt=ONE;cnt<order_cnt;cnt++)
 {
  x=Pred_Coeff[order_cnt][order_cnt]*Pred_Coeff[order_cnt-1][order_cnt-cnt];
  Pred_Coeff[order_cnt][cnt]=Pred_Coeff[order_cnt-ONE][cnt]+x;
 }
 return(0);
}
/******************************************************************************/
/*             Get the new prediction error variance.
*******************************************************************************/
int Get_New_Pred_Var(order_cnt)
int order_cnt;
{
 int cnt;
 double var;
 var=0.0;
 for(cnt=ZERO;cnt<=order_cnt;cnt++)
  var=var+Pred_Coeff[order_cnt][cnt]*Autocor[cnt];
 Pred_Var=var;
 return(0);
}
/******************************************************************************/
/*                     Print the prediction information.
*******************************************************************************/
int Print_Pred_Info(order_cnt)
int order_cnt;
{
 int cnt;
 printf("\t Prediction coefficients for order %d:\n",order_cnt);
 for(cnt=ZERO;cnt<=order_cnt;cnt++)
  printf("\t\t Inx:%d Coeff:%14.12lf\n",cnt,Pred_Coeff[order_cnt][cnt]);
 printf("\n");
 printf("\t Pred. variance of order %d is :%14.12lf.\n",order_cnt,Pred_Var);
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                   Get the receiver coefficient array.
*******************************************************************************/
int Get_Rx_Coeff_Array()
{
 int cnt;
 for(cnt=ONE;cnt<=PRED_MEM_SIZE;cnt++)
  Rx_Coeff_Array[cnt-ONE]=Pred_Coeff[PRED_MEM_SIZE][cnt];
 return(0);
}
/******************************************************************************/
