/******************************************************************************/
#include "p10type.h"
#include "p10ext.h"
/******************************************************************************/
/*                   Get the symbol to be transmitted.
*******************************************************************************/
int Get_Symbol()
{
 Comm_Sym();
 if(Sym_Cnt > ZERO)
  Shift_Into_Err_Buf();
 if(FADING)
  Introduce_Fading();
 else
  Retain_Symbol();
 Up_Tx_Energy();
 return(0);
}
/******************************************************************************/
/*                     Symbols during communication.
*******************************************************************************/
int Comm_Sym()
{
 Get_Bits();
 Inp_Sym_Num=Binary_to_Decimal(Bit,BITS_PER_SYM);
 Mapping();
 Prev_Tx_Sym_Num=Curr_Tx_Sym_Num;
 return(0);
}
/******************************************************************************/
/*                              Get the bits.
*******************************************************************************/
int Get_Bits()
{
 int cnt;
 int random(void);
 for(cnt=ZERO;cnt<BITS_PER_SYM;cnt++)
  Bit[cnt]=random()&01;
 return(0);
}
/******************************************************************************/
/*                               Map to symbol.
*******************************************************************************/
int Mapping()
{
 Curr_Tx_Sym_Num=(Prev_Tx_Sym_Num+Inp_Sym_Num)%CONSTELL_SIZE;
 Re_Sym=Re_Map[Curr_Tx_Sym_Num];
 Im_Sym=Im_Map[Curr_Tx_Sym_Num];
 return(0);
}
/******************************************************************************/
/*                         Shift into error buffer.
*******************************************************************************/
int Shift_Into_Err_Buf()
{
 Err_Buf[Err_Buf_Inx]=Inp_Sym_Num;
 Err_Buf_Inx++;
 if(Err_Buf_Inx == ERR_BUF_SIZE)
  Error_Mes(11);
 return(0);
}
/******************************************************************************/
/*                            Introduce fading.
*******************************************************************************/
int Introduce_Fading()
{
 Get_Cmplx_Variable();
 Get_Modified_Symbol();
 return(0);
}
/******************************************************************************/
/*                    Get the complex random variable.
*******************************************************************************/
int Get_Cmplx_Variable()
{
 int cntd;
 double erand48();
 for(cntd=ZERO;cntd<DIVERSITY;cntd++)
 {
  F_Val[cntd][URADIUS]=erand48(F1_Store[cntd]);
  F_Val[cntd][UTHETA]= erand48(F2_Store[cntd]);
  My_Gauss(F_VARIANCE,F_Val,cntd);
  Get_Re_Fade(cntd);
  Get_Im_Fade(cntd);
  Update_Env(cntd);
 }
 return(0);
}
/******************************************************************************/
/*                            Get Re. fade.
*******************************************************************************/
int Get_Re_Fade(cntd)
int cntd;
{
 int cnt;
 Re_Fade[cntd]=Ip_Coeff*F_Val[cntd][GRAND_0];
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
  Re_Fade[cntd]=Re_Fade[cntd]-Coeff_Array[cnt]*Re_F_Reg[cntd][cnt];
 for(cnt=PRED_MEM_SIZE-ONE;cnt>ZERO;cnt--)
  Re_F_Reg[cntd][cnt]=Re_F_Reg[cntd][cnt-ONE];
 Re_F_Reg[cntd][ZERO]=Re_Fade[cntd];
 return(0);
}
/******************************************************************************/
/*                            Get Im. fade.
*******************************************************************************/
int Get_Im_Fade(cntd)
int cntd;
{
 int cnt;
 Im_Fade[cntd]=Ip_Coeff*F_Val[cntd][GRAND_1];
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
  Im_Fade[cntd]=Im_Fade[cntd]-Coeff_Array[cnt]*Im_F_Reg[cntd][cnt];
 for(cnt=PRED_MEM_SIZE-ONE;cnt>ZERO;cnt--)
  Im_F_Reg[cntd][cnt]=Im_F_Reg[cntd][cnt-ONE];
 Im_F_Reg[cntd][ZERO]=Im_Fade[cntd];
 return(0);
}
/******************************************************************************/
/*         Update the maximum and minimum values of the envelope.
*******************************************************************************/
int Update_Env(cntd)
int cntd;
{
 double mag,re_tmp,im_tmp;
 re_tmp=Re_Fade[cntd]+F_BIAS;
 im_tmp=Im_Fade[cntd]+F_BIAS;
 mag=re_tmp*re_tmp+im_tmp*im_tmp;
 if(mag > Max_Env_Sq[cntd])
  Max_Env_Sq[cntd]=mag;
 else if(mag < Min_Env_Sq[cntd])
  Min_Env_Sq[cntd]=mag;
 return(0);
}
/******************************************************************************/
/*                        Get the modified symbol.
*******************************************************************************/
int Get_Modified_Symbol()
{
 int cntd;
 for(cntd=ZERO;cntd<DIVERSITY;cntd++)
 {
  Rx_Re_Sym[cntd]=Re_Sym*(Re_Fade[cntd]+F_BIAS)-Im_Sym*(Im_Fade[cntd]+F_BIAS);
  Rx_Im_Sym[cntd]=Re_Sym*(Im_Fade[cntd]+F_BIAS)+Im_Sym*(Re_Fade[cntd]+F_BIAS);
 }
 return(0);
}
/******************************************************************************/
/*               Retain the original distortion free symbol.
*******************************************************************************/
int Retain_Symbol()
{
 Rx_Re_Sym[ZERO]=Re_Sym;
 Rx_Im_Sym[ZERO]=Im_Sym;
 return(0);
}
/******************************************************************************/
/*                     Update the transmitted energy.
*******************************************************************************/
int Up_Tx_Energy()
{
 int cntd;
 double energy;
 for(cntd=ZERO;cntd<DIVERSITY;cntd++)
 {
  energy=Rx_Re_Sym[cntd]*Rx_Re_Sym[cntd]+Rx_Im_Sym[cntd]*Rx_Im_Sym[cntd];
  Sym_Energy[cntd]=Sym_Energy[cntd]+energy;
 }
 Sym_Energy_Cnt++;
 return(0);
}
/******************************************************************************/
