/******************************************************************************/
#include <stdio.h>
#include <math.h>
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE PREDICTOR.
*******************************************************************************/
#define PRED_MEM_SIZE 1                      /* Size of the predictor memory. */
/******************************************************************************/
/*                    SIMULATION RUN LENGTH PARAMETER.
*******************************************************************************/
#define VA_DELAY 10                                           /* Delay in VA. */
#define MAX_SYM (10000000+VA_DELAY+PRED_MEM_SIZE)         /* Size of a frame. */
/******************************************************************************/
/*                         CONSTELLATION PARAMETERS.
*******************************************************************************/
#define BITS_PER_SYM 3                                    /* Bits per symbol. */
#define CONSTELL_SIZE 8                             /* Size of constellation. */
/******************************************************************************/
/*                            FADING PARAMETERS.
*******************************************************************************/
#define FADING 1                                      /*1:Fading;0:otherwise. */
#define F_VARIANCE 0.5                   /* Variance of the Gaussian process. */
#define F_BIAS 0.0                                       /* Direct component. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define VARIANCE 0.00800                                   /* Noise variance. */
#define PNS_SIZE 3                     /* Size of array used to generate PNS. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                    RECEIVER REGISTER PARAMETERS.
*******************************************************************************/
#define RX_REG_SIZE 2                       /* Size of the receiver register. */
/******************************************************************************/
/*                         ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define ERR_BUF_SIZE 64                              /* Size of error buffer. */
/******************************************************************************/
/*                    PARAMETERS RELATED TO DIVERSITY.
*******************************************************************************/
#define DIVERSITY 1                             /* No. of diversity channels. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE TRELLIS.
*******************************************************************************/
#define NUM_STATES 8                                /* No. of trellis states. */
#define NUM_INPUTS CONSTELL_SIZE                            /* No. of inputs. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define F_FAC 0.999                                     /* Forgetting factor. */
#define VA_DEPTH VA_DELAY                                 /* Depth of the VA. */
#define MAGNITUDE 2000000.0                   /* Magnitude of initialization. */
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int St_Path[NUM_STATES];                        /* Surviving path of states. */
 int Ip_Path[NUM_STATES];                        /* Surviving path of inputs. */
 double VA_Error[NUM_STATES];                               /* Mag.sq. error. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
