/******************************************************************************/
#include "p5type.h"
#include "p5ext.h"
/******************************************************************************/
/* Calculate the metric associated with each state for the current time
   instant.
*******************************************************************************/
int Calculate_Metric()
{
 for(VA_P_St=ZERO;VA_P_St<NUM_STATES;VA_P_St++)
 {
  for(VA_Inp=ZERO;VA_Inp<NUM_INPUTS;VA_Inp++)
  {
   VA_N_St=Trel_NS[VA_P_St][VA_Inp];
   Get_Curr_Metric();
   Get_Survivor();
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the current branch metric.
*******************************************************************************/
int Get_Curr_Metric()
{
 Data_Demodulation();
 Get_Squared_Pred_Error();
 return(0);
}
/******************************************************************************/
/*                         Do data demodulation.
*******************************************************************************/
int Data_Demodulation()
{
 int cntd,cnt,symnum;
 for(cntd=ZERO;cntd<DIVERSITY;cntd++)
 {
  for(cnt=ONE;cnt<=PRED_MEM_SIZE;cnt++)
  {
   symnum=State_Array[VA_P_St][cnt-ONE];
   Get_Quotient(cntd,cnt,symnum);
  }
  Get_Quotient(cntd,ZERO,VA_Inp);
 }
 return(0);
}
/******************************************************************************/
/*                              Get the quotient.
*******************************************************************************/
int Get_Quotient(cntd,cnt,symnum)
int cntd,cnt,symnum;
{
 double reip,imip,reinvsym,iminvsym;
 reip=Re_Rx_Reg[cntd][cnt];
 imip=Im_Rx_Reg[cntd][cnt];
 reinvsym=Re_Inv_Map[symnum];
 iminvsym=Im_Inv_Map[symnum];
 Re_Data_Dmod[cntd][cnt]=reip*reinvsym-imip*iminvsym;
 Im_Data_Dmod[cntd][cnt]=reip*iminvsym+imip*reinvsym;
 return(0);
}
/******************************************************************************/
/*                      Get the squared prediction error.
*******************************************************************************/
int Get_Squared_Pred_Error()
{
 int cntd,cnt;
 double reerr,imerr;
 Curr_Error_Sq=0.0;
 for(cntd=ZERO;cntd<DIVERSITY;cntd++)
 {
  reerr=Re_Data_Dmod[cntd][ZERO];
  for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
   reerr=reerr+Re_Data_Dmod[cntd][cnt+ONE]*Coeff_Array[cnt];
  imerr=Im_Data_Dmod[cntd][ZERO];
  for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
   imerr=imerr+Im_Data_Dmod[cntd][cnt+ONE]*Coeff_Array[cnt];
  Curr_Error_Sq=Curr_Error_Sq+reerr*reerr+imerr*imerr;
 }
 return(0);
}
/******************************************************************************/
/*               Update the survivor to the next state.
*******************************************************************************/
int Get_Survivor()
{
 Get_New_Wght();
 Get_Existing_Wght();
 if(New_Wght < Old_Wght)
  Survivor_Info();
 return(0);
}
/******************************************************************************/
/*                    Get the new metric at next state.
*******************************************************************************/
int Get_New_Wght()
{
 struct Trellis *prev;
 prev=Lst_Depth->Prv_Depth;
 if((prev!=NULL) && (Sym_Cnt >= RX_REG_SIZE))
  New_Wght=F_FAC*(prev->VA_Error[VA_P_St])+(1.0-F_FAC)*Curr_Error_Sq;
 else
  New_Wght=(1.0-F_FAC)*Curr_Error_Sq;
 /*printf("net new rewght:%lf\n",New_Wght);*/
 return(0);
}
/******************************************************************************/
/*              Get magnitude of existing metric at next state.
*******************************************************************************/
int Get_Existing_Wght()
{
 Old_Wght=Lst_Depth->VA_Error[VA_N_St];
 return(0);
}
/******************************************************************************/
/*                    Get survivor for the next state.
*******************************************************************************/
int Survivor_Info()
{
 Lst_Depth->VA_Error[VA_N_St]=New_Wght;
 Lst_Depth->St_Path[VA_N_St]=VA_P_St;
 Lst_Depth->Ip_Path[VA_N_St]=VA_Inp;
 /*printf("survivor PS:%d inp:%d NS:%d ",VA_P_St,VA_Inp,VA_N_St);
 printf("oldwght:%lf newwght:%lf\n",Old_Wght,New_Wght);
 fflush(stdout);*/
 return(0);
}
/******************************************************************************/
/*           Get the state associated with the minimum metric.
*******************************************************************************/
int Get_Min_Metric()
{
 int cnt;
 double min,cur;
 min= MAGNITUDE;
 for(cnt=ZERO;cnt<NUM_STATES;cnt++)
 {
  cur=Lst_Depth->VA_Error[cnt];
  if(cur < min)
  {
   min=cur;
   Min_Wght_St=cnt;
  }
 }
 return(0);
}
/******************************************************************************/
/*                Go backwards and get the last symbol in time.
*******************************************************************************/
int Get_Last_Symbol()
{
 int tstate,tip;
 struct Trellis *curr;
 curr=Lst_Depth;
 tstate=Min_Wght_St;
 /*printf("Backtracking:\n");*/
 while(curr!=NULL)
 {
  Curr_VA_Op_Sym_Num=Prev_VA_Op_Sym_Num;
  Prev_VA_Op_Sym_Num=tip=curr->Ip_Path[tstate];
  /*printf("currst:%d ip:%d ",tstate,tip);*/
  tstate=curr->St_Path[tstate];
  /*printf("Prevst:%d\n",tstate);*/
  curr=curr->Prv_Depth;
 }
 return(0);
}
/******************************************************************************/
/*                       Reinitialize Linked List.
*******************************************************************************/
int Linked_List_Reinit()
{
 struct Trellis *tnext;
 tnext=Fst_Depth->Nxt_Depth;
 tnext->Prv_Depth=NULL;
 Lst_Depth->Nxt_Depth=Fst_Depth;
 Fst_Depth->Prv_Depth=Lst_Depth;
 Fst_Depth->Nxt_Depth=NULL;
 Lst_Depth=Fst_Depth;
 Fst_Depth=tnext;
 return(0);
}
/******************************************************************************/
/*               Reinitialize metrics at the last depth.
*******************************************************************************/
int Metric_Reinit()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_STATES;cnt++)
  Lst_Depth->VA_Error[cnt]=MAGNITUDE;
 return(0);
}
/******************************************************************************/
