/******************************************************************************/
#include "p7type.h"
#include "p7ext.h"
/******************************************************************************/
int Pre_Processor()
{
 Ck_Var();
 Get_Mapping();
 Print_Mapping();
 Sim_Init();
 return(0);
}
/******************************************************************************/
/*                      Get the mapping to constellation.
*******************************************************************************/
int Get_Mapping()
{
 if(BITS_PER_SYM == 3)
  QAM_08_Mapping();
 else if((BITS_PER_SYM == 4) && (QAM_TYPE == 0))
  QAM_16_0_Mapping();
 else if((BITS_PER_SYM == 4) && (QAM_TYPE == 1))
  QAM_16_1_Mapping();
 return(0);
}
/******************************************************************************/
/*                             8-QAM mapping.
*******************************************************************************/
int QAM_08_Mapping()
{
 int sym_num,rad_num;
 double rad[16],fac1,fac2,phase;
 rad[0]=sqrt(2.0);
 rad[1]=1.0+sqrt(3.0);
 for(sym_num=ZERO;sym_num<CONSTELL_SIZE;sym_num++)
 {
  rad_num=(int)(sym_num/SYM_PER_RADIUS);
  fac1=(double)((rad_num+ONE)%TWO);                      /* Factor for PIBY4. */
  fac2=(double)(sym_num%SYM_PER_RADIUS);                 /* Factor for PIBY2. */
  phase=fac1*PIBY4+fac2*PIBY2;
  Re_Map[sym_num]=rad[rad_num]*cos(phase);
  Im_Map[sym_num]=rad[rad_num]*sin(phase);
 }
 return(0);
}
/******************************************************************************/
/*                        Circular 16-QAM mapping.
*******************************************************************************/
int QAM_16_0_Mapping()
{
 int sym_num,rad_num;
 double rad[16],fac1,fac2,phase;
 rad[0]=sqrt(2.0);
 rad[1]=1.0+sqrt(3.0);
 rad[2]=2.0+sqrt(2.0);
 rad[3]=3.0+sqrt(3.0);
 for(sym_num=ZERO;sym_num<CONSTELL_SIZE;sym_num++)
 {
  rad_num=(int)(sym_num/SYM_PER_RADIUS);
  fac1=(double)(rad_num%TWO);                            /* Factor for PIBY4. */
  fac2=(double)(sym_num%SYM_PER_RADIUS);                 /* Factor for PIBY2. */
  phase=fac1*PIBY4+fac2*PIBY2;
  Re_Map[sym_num]=rad[rad_num]*cos(phase);
  Im_Map[sym_num]=rad[rad_num]*sin(phase);
 }
 return(0);
}
/******************************************************************************/
/*                         Square 16-QAM mapping.
*******************************************************************************/
int QAM_16_1_Mapping()
{
 Get_16_1_Re_Map();
 Get_16_1_Im_Map();
 return(0);
}
/******************************************************************************/
/*                         Get the real mapping.
*******************************************************************************/
int Get_16_1_Re_Map()
{
 Re_Map[0]=   1.0;
 Re_Map[1]=  -1.0;
 Re_Map[2]=  -1.0;
 Re_Map[3]=   1.0;
 Re_Map[4]=  -3.0;
 Re_Map[5]=  -1.0;
 Re_Map[6]=   3.0;
 Re_Map[7]=   1.0;
 Re_Map[8]=  -3.0;
 Re_Map[9]=   3.0;
 Re_Map[10]=  3.0;
 Re_Map[11]= -3.0;
 Re_Map[12]=  1.0;
 Re_Map[13]=  3.0;
 Re_Map[14]= -1.0;
 Re_Map[15]= -3.0;
 return(0);
}
/******************************************************************************/
/*                       Get the imaginary mapping.
*******************************************************************************/
int Get_16_1_Im_Map()
{
 Im_Map[0]=   1.0;
 Im_Map[1]=   1.0;
 Im_Map[2]=  -1.0;
 Im_Map[3]=  -1.0;
 Im_Map[4]=   1.0;
 Im_Map[5]=  -3.0;
 Im_Map[6]=  -1.0;
 Im_Map[7]=   3.0;
 Im_Map[8]=  -3.0;
 Im_Map[9]=  -3.0;
 Im_Map[10]=  3.0;
 Im_Map[11]=  3.0;
 Im_Map[12]= -3.0;
 Im_Map[13]=  1.0;
 Im_Map[14]=  3.0;
 Im_Map[15]= -1.0;
 return(0);
}
/******************************************************************************/
/*                          Print the mapping.
*******************************************************************************/
int Print_Mapping()
{
 int sym_num;
 for(sym_num=ZERO;sym_num<CONSTELL_SIZE;sym_num++)
 {
  printf("\t Symbol:%3d Re:+%16.10lf ",sym_num,Re_Map[sym_num]);
  printf("Im:+%16.10lf\n",Im_Map[sym_num]);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
